@@ -9,18 +9,25 @@ import torch
 from tests import CUDA_DEVICE_COUNT, CUDA_IS_AVAILABLE, MODEL, SOURCE
 from ultralytics import YOLO
 from ultralytics.cfg import TASK2DATA, TASK2MODEL, TASKS
-from ultralytics.utils import ASSETS, WEIGHTS_DIR
+from ultralytics.utils import ASSETS, IS_JETSON, WEIGHTS_DIR
 from ultralytics.utils.autodevice import GPUInfo
 from ultralytics.utils.checks import check_amp
+from ultralytics.utils.torch_utils import TORCH_1_13
 
 # Try to find idle devices if CUDA is available
 DEVICES = []
 if CUDA_IS_AVAILABLE:
-    gpu_info = GPUInfo()
-    gpu_info.print_status()
-    idle_gpus = gpu_info.select_idle_gpu(count=2, min_memory_mb=2048)
-    if idle_gpus:
-        DEVICES = idle_gpus
+    if IS_JETSON:
+        DEVICES = [0]  # NVIDIA Jetson only has one GPU and does not fully support pynvml library
+    else:
+        gpu_info = GPUInfo()
+        gpu_info.print_status()
+        autodevice_fraction = __import__("os").environ.get("YOLO_AUTODEVICE_FRACTION_FREE", 0.3)
+        idle_gpus = gpu_info.select_idle_gpu(
+            count=2, min_memory_fraction=autodevice_fraction, min_util_fraction=autodevice_fraction
+        )
+        if idle_gpus:
+            DEVICES = idle_gpus
 
 
 def test_checks():
@@ -36,6 +43,37 @@ def test_amp():
     assert check_amp(model)
 
 
+@pytest.mark.slow
+@pytest.mark.skipif(not DEVICES, reason="No CUDA devices available")
+@pytest.mark.parametrize(
+    "task, dynamic, int8, half, batch, simplify, nms",
+    [  # generate all combinations except for exclusion cases
+        (task, dynamic, int8, half, batch, simplify, nms)
+        for task, dynamic, int8, half, batch, simplify, nms in product(
+            TASKS, [True, False], [False], [False], [1, 2], [True, False], [True, False]
+        )
+        if not (
+            (int8 and half) or (task == "classify" and nms) or (task == "obb" and nms and (not TORCH_1_13 or IS_JETSON))
+        )
+    ],
+)
+def test_export_onnx_matrix(task, dynamic, int8, half, batch, simplify, nms):
+    """Test YOLO exports to ONNX format with various configurations and parameters."""
+    file = YOLO(TASK2MODEL[task]).export(
+        format="onnx",
+        imgsz=32,
+        dynamic=dynamic,
+        int8=int8,
+        half=half,
+        batch=batch,
+        simplify=simplify,
+        nms=nms,
+        device=DEVICES[0],
+    )
+    YOLO(file)([SOURCE] * batch, imgsz=64 if dynamic else 32, device=DEVICES[0])  # exported model inference
+    Path(file).unlink()  # cleanup
+
+
 @pytest.mark.slow
 @pytest.mark.skipif(True, reason="CUDA export tests disabled pending additional Ultralytics GPU server availability")
 @pytest.mark.skipif(not DEVICES, reason="No CUDA devices available")
@@ -60,7 +98,7 @@ def test_export_engine_matrix(task, dynamic, int8, half, batch):
         batch=batch,
         data=TASK2DATA[task],
         workspace=1,  # reduce workspace GB for less resource utilization during testing
-        simplify=True,  # use 'onnxslim'
+        simplify=True,
         device=DEVICES[0],
     )
     YOLO(file)([SOURCE] * batch, imgsz=64 if dynamic else 32, device=DEVICES[0])  # exported model inference
@@ -71,8 +109,17 @@ def test_export_engine_matrix(task, dynamic, int8, half, batch):
 @pytest.mark.skipif(not DEVICES, reason="No CUDA devices available")
 def test_train():
     """Test model training on a minimal dataset using available CUDA devices."""
-    device = DEVICES if len(DEVICES) > 1 else DEVICES[0]
-    YOLO(MODEL).train(data="coco8.yaml", imgsz=64, epochs=1, device=device)  # requires imgsz>=64
+    import os
+
+    device = tuple(DEVICES) if len(DEVICES) > 1 else DEVICES[0]
+    results = YOLO(MODEL).train(data="coco8.yaml", imgsz=64, epochs=1, device=device)  # requires imgsz>=64
+    # NVIDIA Jetson only has one GPU and therefore skipping checks
+    if not IS_JETSON:
+        visible = eval(os.environ["CUDA_VISIBLE_DEVICES"])
+        assert visible == device, f"Passed GPUs '{device}', but used GPUs '{visible}'"
+        assert (
+            (results is None) if len(DEVICES) > 1 else (results is not None)
+        )  # DDP returns None, single-GPU returns metrics
 
 
 @pytest.mark.slow
