@@ -36,6 +36,7 @@ import platform
 import re
 import threading
 from pathlib import Path
+from typing import Any, Dict, List, Optional, Union
 
 import cv2
 import numpy as np
@@ -43,7 +44,7 @@ import torch
 
 from ultralytics.cfg import get_cfg, get_save_dir
 from ultralytics.data import load_inference_source
-from ultralytics.data.augment import LetterBox, classify_transforms
+from ultralytics.data.augment import LetterBox
 from ultralytics.nn.autobackend import AutoBackend
 from ultralytics.utils import DEFAULT_CFG, LOGGER, MACOS, WINDOWS, callbacks, colorstr, ops
 from ultralytics.utils.checks import check_imgsz, check_imshow
@@ -78,15 +79,15 @@ class BasePredictor:
         data (dict): Data configuration.
         device (torch.device): Device used for prediction.
         dataset (Dataset): Dataset used for prediction.
-        vid_writer (dict): Dictionary of {save_path: video_writer} for saving video output.
-        plotted_img (numpy.ndarray): Last plotted image.
+        vid_writer (Dict[str, cv2.VideoWriter]): Dictionary of {save_path: video_writer} for saving video output.
+        plotted_img (np.ndarray): Last plotted image.
         source_type (SimpleNamespace): Type of input source.
         seen (int): Number of images processed.
-        windows (list): List of window names for visualization.
+        windows (List[str]): List of window names for visualization.
         batch (tuple): Current batch data.
-        results (list): Current batch results.
+        results (List[Any]): Current batch results.
         transforms (callable): Image transforms for classification.
-        callbacks (dict): Callback functions for different events.
+        callbacks (Dict[str, List[callable]]): Callback functions for different events.
         txt_path (Path): Path to save text results.
         _lock (threading.Lock): Lock for thread-safe inference.
 
@@ -105,14 +106,19 @@ class BasePredictor:
         add_callback: Register a new callback function.
     """
 
-    def __init__(self, cfg=DEFAULT_CFG, overrides=None, _callbacks=None):
+    def __init__(
+        self,
+        cfg=DEFAULT_CFG,
+        overrides: Optional[Dict[str, Any]] = None,
+        _callbacks: Optional[Dict[str, List[callable]]] = None,
+    ):
         """
         Initialize the BasePredictor class.
 
         Args:
             cfg (str | dict): Path to a configuration file or a configuration dictionary.
-            overrides (dict | None): Configuration overrides.
-            _callbacks (dict | None): Dictionary of callback functions.
+            overrides (dict, optional): Configuration overrides.
+            _callbacks (dict, optional): Dictionary of callback functions.
         """
         self.args = get_cfg(cfg, overrides)
         self.save_dir = get_save_dir(self.args)
@@ -141,12 +147,15 @@ class BasePredictor:
         self._lock = threading.Lock()  # for automatic thread-safe inference
         callbacks.add_integration_callbacks(self)
 
-    def preprocess(self, im):
+    def preprocess(self, im: Union[torch.Tensor, List[np.ndarray]]) -> torch.Tensor:
         """
-        Prepares input image before inference.
+        Prepare input image before inference.
 
         Args:
-            im (torch.Tensor | List(np.ndarray)): Images of shape (N, 3, h, w) for tensor, [(h, w, 3) x N] for list.
+            im (torch.Tensor | List[np.ndarray]): Images of shape (N, 3, H, W) for tensor, [(H, W, 3) x N] for list.
+
+        Returns:
+            (torch.Tensor): Preprocessed image tensor of shape (N, 3, H, W).
         """
         not_tensor = not isinstance(im, torch.Tensor)
         if not_tensor:
@@ -163,7 +172,7 @@ class BasePredictor:
             im /= 255  # 0 - 255 to 0.0 - 1.0
         return im
 
-    def inference(self, im, *args, **kwargs):
+    def inference(self, im: torch.Tensor, *args, **kwargs):
         """Run inference on a given image using the specified model and arguments."""
         visualize = (
             increment_path(self.save_dir / Path(self.batch[0][0]).stem, mkdir=True)
@@ -172,15 +181,15 @@ class BasePredictor:
         )
         return self.model(im, augment=self.args.augment, visualize=visualize, embed=self.args.embed, *args, **kwargs)
 
-    def pre_transform(self, im):
+    def pre_transform(self, im: List[np.ndarray]) -> List[np.ndarray]:
         """
         Pre-transform input image before inference.
 
         Args:
-            im (List[np.ndarray]): Images of shape (N, 3, h, w) for tensor, [(h, w, 3) x N] for list.
+            im (List[np.ndarray]): List of images with shape [(H, W, 3) x N].
 
         Returns:
-            (List[np.ndarray]): A list of transformed images.
+            (List[np.ndarray]): List of transformed images.
         """
         same_shapes = len({x.shape for x in im}) == 1
         letterbox = LetterBox(
@@ -196,14 +205,14 @@ class BasePredictor:
         """Post-process predictions for an image and return them."""
         return preds
 
-    def __call__(self, source=None, model=None, stream=False, *args, **kwargs):
+    def __call__(self, source=None, model=None, stream: bool = False, *args, **kwargs):
         """
         Perform inference on an image or stream.
 
         Args:
-            source (str | Path | List[str] | List[Path] | List[np.ndarray] | np.ndarray | torch.Tensor | None):
+            source (str | Path | List[str] | List[Path] | List[np.ndarray] | np.ndarray | torch.Tensor, optional):
                 Source for inference.
-            model (str | Path | torch.nn.Module | None): Model for inference.
+            model (str | Path | torch.nn.Module, optional): Model for inference.
             stream (bool): Whether to stream the inference results. If True, returns a generator.
             *args (Any): Additional arguments for the inference method.
             **kwargs (Any): Additional keyword arguments for the inference method.
@@ -226,9 +235,9 @@ class BasePredictor:
         generator without storing results.
 
         Args:
-            source (str | Path | List[str] | List[Path] | List[np.ndarray] | np.ndarray | torch.Tensor | None):
+            source (str | Path | List[str] | List[Path] | List[np.ndarray] | np.ndarray | torch.Tensor, optional):
                 Source for inference.
-            model (str | Path | torch.nn.Module | None): Model for inference.
+            model (str | Path | torch.nn.Module, optional): Model for inference.
 
         Note:
             Do not modify this function or remove the generator. The generator ensures that no outputs are
@@ -247,20 +256,12 @@ class BasePredictor:
                 Source for inference.
         """
         self.imgsz = check_imgsz(self.args.imgsz, stride=self.model.stride, min_dim=2)  # check image size
-        self.transforms = (
-            getattr(
-                self.model.model,
-                "transforms",
-                classify_transforms(self.imgsz[0]),
-            )
-            if self.args.task == "classify"
-            else None
-        )
         self.dataset = load_inference_source(
             source=source,
             batch=self.args.batch,
             vid_stride=self.args.vid_stride,
             buffer=self.args.stream_buffer,
+            channels=getattr(self.model, "ch", 3),
         )
         self.source_type = self.dataset.source_type
         if not getattr(self, "stream", True) and (
@@ -278,9 +279,9 @@ class BasePredictor:
         Stream real-time inference on camera feed and save results to file.
 
         Args:
-            source (str | Path | List[str] | List[Path] | List[np.ndarray] | np.ndarray | torch.Tensor | None):
+            source (str | Path | List[str] | List[Path] | List[np.ndarray] | np.ndarray | torch.Tensor, optional):
                 Source for inference.
-            model (str | Path | torch.nn.Module | None): Model for inference.
+            model (str | Path | torch.nn.Module, optional): Model for inference.
             *args (Any): Additional arguments for the inference method.
             **kwargs (Any): Additional keyword arguments for the inference method.
 
@@ -373,12 +374,12 @@ class BasePredictor:
             LOGGER.info(f"Results saved to {colorstr('bold', self.save_dir)}{s}")
         self.run_callbacks("on_predict_end")
 
-    def setup_model(self, model, verbose=True):
+    def setup_model(self, model, verbose: bool = True):
         """
         Initialize YOLO model with given parameters and set it to evaluation mode.
 
         Args:
-            model (str | Path | torch.nn.Module | None): Model to load or use.
+            model (str | Path | torch.nn.Module, optional): Model to load or use.
             verbose (bool): Whether to print verbose output.
         """
         self.model = AutoBackend(
@@ -394,9 +395,11 @@ class BasePredictor:
 
         self.device = self.model.device  # update device
         self.args.half = self.model.fp16  # update half
+        if hasattr(self.model, "imgsz"):
+            self.args.imgsz = self.model.imgsz  # reuse imgsz from export metadata
         self.model.eval()
 
-    def write_results(self, i, p, im, s):
+    def write_results(self, i: int, p: Path, im: torch.Tensor, s: List[str]) -> str:
         """
         Write inference results to a file or directory.
 
@@ -447,7 +450,7 @@ class BasePredictor:
 
         return string
 
-    def save_predicted_images(self, save_path="", frame=0):
+    def save_predicted_images(self, save_path: str = "", frame: int = 0):
         """
         Save video predictions as mp4 or images as jpg at specified path.
 
@@ -481,7 +484,7 @@ class BasePredictor:
         else:
             cv2.imwrite(str(Path(save_path).with_suffix(".jpg")), im)  # save to JPG for best support
 
-    def show(self, p=""):
+    def show(self, p: str = ""):
         """Display an image in a window."""
         im = self.plotted_img
         if platform.system() == "Linux" and p not in self.windows:
@@ -496,6 +499,6 @@ class BasePredictor:
         for callback in self.callbacks.get(event, []):
             callback(self)
 
-    def add_callback(self, event: str, func):
+    def add_callback(self, event: str, func: callable):
         """Add a callback function for a specific event."""
         self.callbacks[event].append(func)
