@@ -4,6 +4,7 @@ import cv2
 import torch
 from PIL import Image
 
+from ultralytics.data.augment import classify_transforms
 from ultralytics.engine.predictor import BasePredictor
 from ultralytics.engine.results import Results
 from ultralytics.utils import DEFAULT_CFG, ops
@@ -43,7 +44,7 @@ class ClassificationPredictor(BasePredictor):
         tasks. It ensures the task is set to 'classify' regardless of input configuration.
 
         Args:
-            cfg (dict): Default configuration dictionary containing prediction settings. Defaults to DEFAULT_CFG.
+            cfg (dict): Default configuration dictionary containing prediction settings.
             overrides (dict, optional): Configuration overrides that take precedence over cfg.
             _callbacks (list, optional): List of callback functions to be executed during prediction.
         """
@@ -51,20 +52,30 @@ class ClassificationPredictor(BasePredictor):
         self.args.task = "classify"
         self._legacy_transform_name = "ultralytics.yolo.data.augment.ToTensor"
 
+    def setup_source(self, source):
+        """Set up source and inference mode and classify transforms."""
+        super().setup_source(source)
+        updated = (
+            self.model.model.transforms.transforms[0].size != max(self.imgsz)
+            if hasattr(self.model.model, "transforms") and hasattr(self.model.model.transforms.transforms[0], "size")
+            else True
+        )
+        self.transforms = self.model.model.transforms if not updated else classify_transforms(self.imgsz)
+
     def preprocess(self, img):
         """Convert input images to model-compatible tensor format with appropriate normalization."""
         if not isinstance(img, torch.Tensor):
             is_legacy_transform = any(
                 self._legacy_transform_name in str(transform) for transform in self.transforms.transforms
             )
-            if is_legacy_transform:  # to handle legacy transforms
+            if is_legacy_transform:  # Handle legacy transforms
                 img = torch.stack([self.transforms(im) for im in img], dim=0)
             else:
                 img = torch.stack(
                     [self.transforms(Image.fromarray(cv2.cvtColor(im, cv2.COLOR_BGR2RGB))) for im in img], dim=0
                 )
         img = (img if isinstance(img, torch.Tensor) else torch.from_numpy(img)).to(self.model.device)
-        return img.half() if self.model.fp16 else img.float()  # uint8 to fp16/32
+        return img.half() if self.model.fp16 else img.float()  # Convert uint8 to fp16/32
 
     def postprocess(self, preds, img, orig_imgs):
         """
@@ -78,7 +89,7 @@ class ClassificationPredictor(BasePredictor):
         Returns:
             (List[Results]): List of Results objects containing classification results for each image.
         """
-        if not isinstance(orig_imgs, list):  # input images are a torch.Tensor, not a list
+        if not isinstance(orig_imgs, list):  # Input images are a torch.Tensor, not a list
             orig_imgs = ops.convert_torch2numpy_batch(orig_imgs)
 
         preds = preds[0] if isinstance(preds, (list, tuple)) else preds
