@@ -1,6 +1,7 @@
 # Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
 import json
+from typing import Any, List, Tuple
 
 import cv2
 import numpy as np
@@ -33,13 +34,13 @@ class ParkingPtsSelection:
         canvas_max_height (int): Maximum height of the canvas.
 
     Methods:
-        initialize_properties: Initializes the necessary properties.
-        upload_image: Uploads an image, resizes it to fit the canvas, and displays it.
-        on_canvas_click: Handles mouse clicks to add points for bounding boxes.
-        draw_box: Draws a bounding box on the canvas.
-        remove_last_bounding_box: Removes the last bounding box and redraws the canvas.
-        redraw_canvas: Redraws the canvas with the image and all bounding boxes.
-        save_to_json: Saves the bounding boxes to a JSON file.
+        initialize_properties: Initialize properties for image, canvas, bounding boxes, and dimensions.
+        upload_image: Upload and display an image on the canvas, resizing it to fit within specified dimensions.
+        on_canvas_click: Handle mouse clicks to add points for bounding boxes on the canvas.
+        draw_box: Draw a bounding box on the canvas using the provided coordinates.
+        remove_last_bounding_box: Remove the last bounding box from the list and redraw the canvas.
+        redraw_canvas: Redraw the canvas with the image and all bounding boxes.
+        save_to_json: Save the selected parking zone points to a JSON file with scaled coordinates.
 
     Examples:
         >>> parking_selector = ParkingPtsSelection()
@@ -48,7 +49,7 @@ class ParkingPtsSelection:
 
     def __init__(self):
         """Initialize the ParkingPtsSelection class, setting up UI and properties for parking zone point selection."""
-        try:  # check if tkinter installed
+        try:  # Check if tkinter is installed
             import tkinter as tk
             from tkinter import filedialog, messagebox
         except ImportError:  # Display error with recommendations
@@ -68,19 +69,19 @@ class ParkingPtsSelection:
             return
 
         self.tk, self.filedialog, self.messagebox = tk, filedialog, messagebox
-        self.master = self.tk.Tk()  # Reference to the main application window or parent widget
+        self.master = self.tk.Tk()  # Reference to the main application window
         self.master.title("Ultralytics Parking Zones Points Selector")
         self.master.resizable(False, False)
 
-        self.canvas = self.tk.Canvas(self.master, bg="white")  # Canvas widget for displaying images or graphics
+        self.canvas = self.tk.Canvas(self.master, bg="white")  # Canvas widget for displaying images
         self.canvas.pack(side=self.tk.BOTTOM)
 
         self.image = None  # Variable to store the loaded image
         self.canvas_image = None  # Reference to the image displayed on the canvas
         self.canvas_max_width = None  # Maximum allowed width for the canvas
         self.canvas_max_height = None  # Maximum allowed height for the canvas
-        self.rg_data = None  # Data related to region or annotation management
-        self.current_box = None  # Stores the currently selected or active bounding box
+        self.rg_data = None  # Data for region annotation management
+        self.current_box = None  # Stores the currently selected bounding box
         self.imgh = None  # Height of the current image
         self.imgw = None  # Width of the current image
 
@@ -107,7 +108,7 @@ class ParkingPtsSelection:
 
     def upload_image(self):
         """Upload and display an image on the canvas, resizing it to fit within specified dimensions."""
-        from PIL import Image, ImageTk  # scope because ImageTk requires tkinter package
+        from PIL import Image, ImageTk  # Scoped import because ImageTk requires tkinter package
 
         self.image = Image.open(self.filedialog.askopenfilename(filetypes=[("Image Files", "*.png *.jpg *.jpeg")]))
         if not self.image:
@@ -138,7 +139,7 @@ class ParkingPtsSelection:
             self.draw_box(self.current_box)
             self.current_box.clear()
 
-    def draw_box(self, box):
+    def draw_box(self, box: List[Tuple[int, int]]):
         """Draw a bounding box on the canvas using the provided coordinates."""
         for i in range(4):
             self.canvas.create_line(box[i], box[(i + 1) % 4], fill="blue", width=2)
@@ -163,7 +164,7 @@ class ParkingPtsSelection:
         scale_w, scale_h = self.imgw / self.canvas.winfo_width(), self.imgh / self.canvas.winfo_height()
         data = [{"points": [(int(x * scale_w), int(y * scale_h)) for x, y in box]} for box in self.rg_data]
 
-        from io import StringIO  # Function level import, as it's only required to store coordinates, not every frame
+        from io import StringIO  # Function level import, as it's only required to store coordinates
 
         write_buffer = StringIO()
         json.dump(data, write_buffer, indent=4)
@@ -188,7 +189,7 @@ class ParkingManagement(BaseSolution):
         dc (Tuple[int, int, int]): RGB color tuple for centroid visualization of detected objects.
 
     Methods:
-        process: Processes the input image for parking lot management and visualization.
+        process: Process the input image for parking lot management and visualization.
 
     Examples:
         >>> from ultralytics.solutions import ParkingManagement
@@ -197,7 +198,7 @@ class ParkingManagement(BaseSolution):
         >>> print(f"Available spaces: {parking_manager.pr_info['Available']}")
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: Any):
         """Initialize the parking management system with a YOLO model and visualization settings."""
         super().__init__(**kwargs)
 
@@ -209,13 +210,13 @@ class ParkingManagement(BaseSolution):
         with open(self.json_file) as f:
             self.json = json.load(f)
 
-        self.pr_info = {"Occupancy": 0, "Available": 0}  # dictionary for parking information
+        self.pr_info = {"Occupancy": 0, "Available": 0}  # Dictionary for parking information
 
-        self.arc = (0, 0, 255)  # available region color
-        self.occ = (0, 255, 0)  # occupied region color
-        self.dc = (255, 0, 189)  # centroid color for each box
+        self.arc = (0, 0, 255)  # Available region color
+        self.occ = (0, 255, 0)  # Occupied region color
+        self.dc = (255, 0, 189)  # Centroid color for each box
 
-    def process(self, im0):
+    def process(self, im0: np.ndarray) -> SolutionResults:
         """
         Process the input image for parking lot management and visualization.
 
@@ -235,14 +236,14 @@ class ParkingManagement(BaseSolution):
             >>> image = cv2.imread("parking_lot.jpg")
             >>> results = parking_manager.process(image)
         """
-        self.extract_tracks(im0)  # extract tracks from im0
-        es, fs = len(self.json), 0  # empty slots, filled slots
-        annotator = SolutionAnnotator(im0, self.line_width)  # init annotator
+        self.extract_tracks(im0)  # Extract tracks from im0
+        es, fs = len(self.json), 0  # Empty slots, filled slots
+        annotator = SolutionAnnotator(im0, self.line_width)  # Initialize annotator
 
         for region in self.json:
             # Convert points to a NumPy array with the correct dtype and reshape properly
             pts_array = np.array(region["points"], dtype=np.int32).reshape((-1, 1, 2))
-            rg_occupied = False  # occupied region initialization
+            rg_occupied = False  # Occupied region initialization
             for box, cls in zip(self.boxes, self.clss):
                 xc, yc = int((box[0] + box[2]) / 2), int((box[1] + box[3]) / 2)
                 dist = cv2.pointPolygonTest(pts_array, (xc, yc), False)
@@ -254,7 +255,7 @@ class ParkingManagement(BaseSolution):
                     rg_occupied = True
                     break
             fs, es = (fs + 1, es - 1) if rg_occupied else (fs, es)
-            # Plotting regions
+            # Plot regions
             cv2.polylines(im0, [pts_array], isClosed=True, color=self.occ if rg_occupied else self.arc, thickness=2)
 
         self.pr_info["Occupancy"], self.pr_info["Available"] = fs, es
@@ -262,7 +263,7 @@ class ParkingManagement(BaseSolution):
         annotator.display_analytics(im0, self.pr_info, (104, 31, 17), (255, 255, 255), 10)
 
         plot_im = annotator.result()
-        self.display_output(plot_im)  # display output with base class function
+        self.display_output(plot_im)  # Display output with base class function
 
         # Return SolutionResults
         return SolutionResults(
