@@ -55,14 +55,11 @@ def on_pretrain_routine_end(trainer):
     Args:
         trainer (ultralytics.engine.trainer.BaseTrainer): The training object with arguments and parameters to log.
 
-    Global:
-        mlflow: The imported mlflow module to use for logging.
-
     Environment Variables:
         MLFLOW_TRACKING_URI: The URI for MLflow tracking. If not set, defaults to 'runs/mlflow'.
         MLFLOW_EXPERIMENT_NAME: The name of the MLflow experiment. If not set, defaults to trainer.args.project.
         MLFLOW_RUN: The name of the MLflow run. If not set, defaults to trainer.args.name.
-        MLFLOW_KEEP_RUN_ACTIVE: Boolean indicating whether to keep the MLflow run active after the end of training.
+        MLFLOW_KEEP_RUN_ACTIVE: Boolean indicating whether to keep the MLflow run active after training ends.
     """
     global mlflow
 
@@ -107,7 +104,7 @@ def on_fit_epoch_end(trainer):
 
 
 def on_train_end(trainer):
-    """Log model artifacts at the end of the training."""
+    """Log model artifacts at the end of training."""
     if not mlflow:
         return
     mlflow.log_artifact(str(trainer.best.parent))  # log save_dir/weights directory with best.pt and last.pt
