@@ -1,5 +1,6 @@
 # Ultralytics 🚀 AGPL-3.0 License - https://ultralytics.com/license
 
+import functools
 import gc
 import math
 import os
@@ -48,7 +49,7 @@ if WINDOWS and check_version(torch.__version__, "==2.4.0"):  # reject version 2.
 
 @contextmanager
 def torch_distributed_zero_first(local_rank: int):
-    """Ensures all processes in distributed training wait for the local master (rank 0) to complete a task first."""
+    """Ensure all processes in distributed training wait for the local master (rank 0) to complete a task first."""
     initialized = dist.is_available() and dist.is_initialized()
     use_ids = initialized and dist.get_backend() == "nccl"
 
@@ -60,10 +61,10 @@ def torch_distributed_zero_first(local_rank: int):
 
 
 def smart_inference_mode():
-    """Applies torch.inference_mode() decorator if torch>=1.9.0 else torch.no_grad() decorator."""
+    """Apply torch.inference_mode() decorator if torch>=1.9.0 else torch.no_grad() decorator."""
 
     def decorate(fn):
-        """Applies appropriate torch decorator for inference mode based on torch version."""
+        """Apply appropriate torch decorator for inference mode based on torch version."""
         if TORCH_1_9 and torch.is_inference_mode_enabled():
             return fn  # already in inference_mode, act as a pass-through
         else:
@@ -81,7 +82,7 @@ def autocast(enabled: bool, device: str = "cuda"):
 
     Args:
         enabled (bool): Whether to enable automatic mixed precision.
-        device (str, optional): The device to use for autocast. Defaults to 'cuda'.
+        device (str, optional): The device to use for autocast.
 
     Returns:
         (torch.amp.autocast): The appropriate autocast context manager.
@@ -101,6 +102,7 @@ def autocast(enabled: bool, device: str = "cuda"):
         return torch.cuda.amp.autocast(enabled)
 
 
+@functools.lru_cache
 def get_cpu_info():
     """Return a string with system CPU information, i.e. 'Apple M2'."""
     from ultralytics.utils import PERSISTENT_CACHE  # avoid circular import error
@@ -118,6 +120,7 @@ def get_cpu_info():
     return PERSISTENT_CACHE.get("cpu_info", "unknown")
 
 
+@functools.lru_cache
 def get_gpu_info(index):
     """Return a string with system GPU information, i.e. 'Tesla T4, 15102MiB'."""
     properties = torch.cuda.get_device_properties(index)
@@ -133,9 +136,8 @@ def select_device(device="", batch=0, newline=False, verbose=True):
     exception if the requested device(s) are not available.
 
     Args:
-        device (str | torch.device, optional): Device string or torch.device object.
-            Options are 'None', 'cpu', or 'cuda', or '0' or '0,1,2,3'. Defaults to an empty string, which auto-selects
-            the first available GPU, or CPU if no GPU is available.
+        device (str | torch.device, optional): Device string or torch.device object. Options are 'None', 'cpu', or
+            'cuda', or '0' or '0,1,2,3'. Auto-selects the first available GPU, or CPU if no GPU is available.
         batch (int, optional): Batch size being used in your model.
         newline (bool, optional): If True, adds a newline at the end of the log string.
         verbose (bool, optional): If True, logs the device information.
@@ -154,7 +156,7 @@ def select_device(device="", batch=0, newline=False, verbose=True):
         >>> select_device("cpu")
         device(type='cpu')
 
-    Note:
+    Notes:
         Sets the 'CUDA_VISIBLE_DEVICES' environment variable for specifying which GPUs to use.
     """
     if isinstance(device, torch.device) or str(device).startswith(("tpu", "intel")):
@@ -171,7 +173,7 @@ def select_device(device="", batch=0, newline=False, verbose=True):
 
         # Replace each -1 with a selected GPU or remove it
         parts = device.split(",")
-        selected = GPUInfo().select_idle_gpu(count=parts.count("-1"), min_memory_mb=2048)
+        selected = GPUInfo().select_idle_gpu(count=parts.count("-1"), min_memory_fraction=0.2)
         for i in range(len(parts)):
             if parts[i] == "-1":
                 parts[i] = str(selected.pop(0)) if selected else ""
@@ -240,7 +242,7 @@ def select_device(device="", batch=0, newline=False, verbose=True):
 
 
 def time_sync():
-    """PyTorch-accurate time."""
+    """Return PyTorch-accurate time."""
     if torch.cuda.is_available():
         torch.cuda.synchronize()
     return time.time()
@@ -317,12 +319,15 @@ def model_info(model, detailed=False, verbose=True, imgsz=640):
 
     Args:
         model (nn.Module): Model to analyze.
-        detailed (bool, optional): Whether to print detailed layer information. Defaults to False.
-        verbose (bool, optional): Whether to print model information. Defaults to True.
-        imgsz (int | List, optional): Input image size. Defaults to 640.
+        detailed (bool, optional): Whether to print detailed layer information.
+        verbose (bool, optional): Whether to print model information.
+        imgsz (int | list, optional): Input image size.
 
     Returns:
-        (Tuple[int, int, int, float]): Number of layers, parameters, gradients, and GFLOPs.
+        n_l (int): Number of layers.
+        n_p (int): Number of parameters.
+        n_g (int): Number of gradients.
+        flops (float): GFLOPs.
     """
     if not verbose:
         return
@@ -407,7 +412,7 @@ def get_flops(model, imgsz=640):
 
     Args:
         model (nn.Module): The model to calculate FLOPs for.
-        imgsz (int | List[int], optional): Input image size. Defaults to 640.
+        imgsz (int | list, optional): Input image size.
 
     Returns:
         (float): The model FLOPs in billions.
@@ -445,7 +450,7 @@ def get_flops_with_torch_profiler(model, imgsz=640):
 
     Args:
         model (nn.Module): The model to calculate FLOPs for.
-        imgsz (int | List[int], optional): Input image size. Defaults to 640.
+        imgsz (int | list, optional): Input image size.
 
     Returns:
         (float): The model's FLOPs in billions.
@@ -488,13 +493,13 @@ def initialize_weights(model):
 
 def scale_img(img, ratio=1.0, same_shape=False, gs=32):
     """
-    Scales and pads an image tensor, optionally maintaining aspect ratio and padding to gs multiple.
+    Scale and pad an image tensor, optionally maintaining aspect ratio and padding to gs multiple.
 
     Args:
         img (torch.Tensor): Input image tensor.
-        ratio (float, optional): Scaling ratio. Defaults to 1.0.
-        same_shape (bool, optional): Whether to maintain the same shape. Defaults to False.
-        gs (int, optional): Grid size for padding. Defaults to 32.
+        ratio (float, optional): Scaling ratio.
+        same_shape (bool, optional): Whether to maintain the same shape.
+        gs (int, optional): Grid size for padding.
 
     Returns:
         (torch.Tensor): Scaled and padded image tensor.
@@ -511,13 +516,13 @@ def scale_img(img, ratio=1.0, same_shape=False, gs=32):
 
 def copy_attr(a, b, include=(), exclude=()):
     """
-    Copies attributes from object 'b' to object 'a', with options to include/exclude certain attributes.
+    Copy attributes from object 'b' to object 'a', with options to include/exclude certain attributes.
 
     Args:
-        a (object): Destination object to copy attributes to.
-        b (object): Source object to copy attributes from.
-        include (tuple, optional): Attributes to include. If empty, all attributes are included. Defaults to ().
-        exclude (tuple, optional): Attributes to exclude. Defaults to ().
+        a (Any): Destination object to copy attributes to.
+        b (Any): Source object to copy attributes from.
+        include (tuple, optional): Attributes to include. If empty, all attributes are included.
+        exclude (tuple, optional): Attributes to exclude.
     """
     for k, v in b.__dict__.items():
         if (len(include) and k not in include) or k.startswith("_") or k in exclude:
@@ -543,12 +548,12 @@ def get_latest_opset():
 
 def intersect_dicts(da, db, exclude=()):
     """
-    Returns a dictionary of intersecting keys with matching shapes, excluding 'exclude' keys, using da values.
+    Return a dictionary of intersecting keys with matching shapes, excluding 'exclude' keys, using da values.
 
     Args:
         da (dict): First dictionary.
         db (dict): Second dictionary.
-        exclude (tuple, optional): Keys to exclude. Defaults to ().
+        exclude (tuple, optional): Keys to exclude.
 
     Returns:
         (dict): Dictionary of intersecting keys with matching shapes.
@@ -558,7 +563,7 @@ def intersect_dicts(da, db, exclude=()):
 
 def is_parallel(model):
     """
-    Returns True if model is of type DP or DDP.
+    Return True if model is of type DP or DDP.
 
     Args:
         model (nn.Module): Model to check.
@@ -571,7 +576,7 @@ def is_parallel(model):
 
 def de_parallel(model):
     """
-    De-parallelize a model: returns single-GPU model if model is of type DP or DDP.
+    De-parallelize a model: return single-GPU model if model is of type DP or DDP.
 
     Args:
         model (nn.Module): Model to de-parallelize.
@@ -584,12 +589,12 @@ def de_parallel(model):
 
 def one_cycle(y1=0.0, y2=1.0, steps=100):
     """
-    Returns a lambda function for sinusoidal ramp from y1 to y2 https://arxiv.org/pdf/1812.01187.pdf.
+    Return a lambda function for sinusoidal ramp from y1 to y2 https://arxiv.org/pdf/1812.01187.pdf.
 
     Args:
-        y1 (float, optional): Initial value. Defaults to 0.0.
-        y2 (float, optional): Final value. Defaults to 1.0.
-        steps (int, optional): Number of steps. Defaults to 100.
+        y1 (float, optional): Initial value.
+        y2 (float, optional): Final value.
+        steps (int, optional): Number of steps.
 
     Returns:
         (function): Lambda function for computing the sinusoidal ramp.
@@ -602,8 +607,8 @@ def init_seeds(seed=0, deterministic=False):
     Initialize random number generator (RNG) seeds https://pytorch.org/docs/stable/notes/randomness.html.
 
     Args:
-        seed (int, optional): Random seed. Defaults to 0.
-        deterministic (bool, optional): Whether to set deterministic algorithms. Defaults to False.
+        seed (int, optional): Random seed.
+        deterministic (bool, optional): Whether to set deterministic algorithms.
     """
     random.seed(seed)
     np.random.seed(seed)
@@ -624,7 +629,7 @@ def init_seeds(seed=0, deterministic=False):
 
 
 def unset_deterministic():
-    """Unsets all the configurations applied for deterministic training."""
+    """Unset all the configurations applied for deterministic training."""
     torch.use_deterministic_algorithms(False)
     torch.backends.cudnn.deterministic = False
     os.environ.pop("CUBLAS_WORKSPACE_CONFIG", None)
@@ -657,9 +662,9 @@ class ModelEMA:
 
         Args:
             model (nn.Module): Model to create EMA for.
-            decay (float, optional): Maximum EMA decay rate. Defaults to 0.9999.
-            tau (int, optional): EMA decay time constant. Defaults to 2000.
-            updates (int, optional): Initial number of updates. Defaults to 0.
+            decay (float, optional): Maximum EMA decay rate.
+            tau (int, optional): EMA decay time constant.
+            updates (int, optional): Initial number of updates.
         """
         self.ema = deepcopy(de_parallel(model)).eval()  # FP32 EMA
         self.updates = updates  # number of EMA updates
@@ -688,12 +693,12 @@ class ModelEMA:
 
     def update_attr(self, model, include=(), exclude=("process_group", "reducer")):
         """
-        Updates attributes and saves stripped model with optimizer removed.
+        Update attributes and save stripped model with optimizer removed.
 
         Args:
             model (nn.Module): Model to update attributes from.
-            include (tuple, optional): Attributes to include. Defaults to ().
-            exclude (tuple, optional): Attributes to exclude. Defaults to ("process_group", "reducer").
+            include (tuple, optional): Attributes to include.
+            exclude (tuple, optional): Attributes to exclude.
         """
         if self.enabled:
             copy_attr(self.ema, model, include, exclude)
@@ -704,8 +709,9 @@ def strip_optimizer(f: Union[str, Path] = "best.pt", s: str = "", updates: dict
     Strip optimizer from 'f' to finalize training, optionally save as 's'.
 
     Args:
-        f (str | Path): File path to model to strip the optimizer from. Defaults to 'best.pt'.
-        s (str, optional): File path to save the model with stripped optimizer to. If not provided, 'f' will be overwritten.
+        f (str | Path): File path to model to strip the optimizer from.
+        s (str, optional): File path to save the model with stripped optimizer to. If not provided, 'f' will be
+            overwritten.
         updates (dict, optional): A dictionary of updates to overlay onto the checkpoint before saving.
 
     Returns:
@@ -761,7 +767,7 @@ def strip_optimizer(f: Union[str, Path] = "best.pt", s: str = "", updates: dict
 
 def convert_optimizer_state_dict_to_fp16(state_dict):
     """
-    Converts the state_dict of a given optimizer to FP16, focusing on the 'state' key for tensor conversions.
+    Convert the state_dict of a given optimizer to FP16, focusing on the 'state' key for tensor conversions.
 
     Args:
         state_dict (dict): Optimizer state dictionary.
@@ -787,7 +793,7 @@ def cuda_memory_usage(device=None):
     Finally, it updates the dictionary with the amount of memory reserved by CUDA on the specified device.
 
     Args:
-        device (torch.device, optional): The CUDA device to query memory usage for. Defaults to None.
+        device (torch.device, optional): The CUDA device to query memory usage for.
 
     Yields:
         (dict): A dictionary with a key 'memory' initialized to 0, which will be updated with the reserved memory.
@@ -808,11 +814,11 @@ def profile_ops(input, ops, n=10, device=None, max_num_obj=0):
     Ultralytics speed, memory and FLOPs profiler.
 
     Args:
-        input (torch.Tensor | List[torch.Tensor]): Input tensor(s) to profile.
-        ops (nn.Module | List[nn.Module]): Model or list of operations to profile.
-        n (int, optional): Number of iterations to average. Defaults to 10.
-        device (str | torch.device, optional): Device to profile on. Defaults to None.
-        max_num_obj (int, optional): Maximum number of objects for simulation. Defaults to 0.
+        input (torch.Tensor | list): Input tensor(s) to profile.
+        ops (nn.Module | list): Model or list of operations to profile.
+        n (int, optional): Number of iterations to average.
+        device (str | torch.device, optional): Device to profile on.
+        max_num_obj (int, optional): Maximum number of objects for simulation.
 
     Returns:
         (list): Profile results for each operation.
